@@ -1,14 +1,17 @@
 /** dec-templ.c  -  XML decoder template, used in code generation
- ** Copyright (c) 2006 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
+ ** Copyright (c) 2006-2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ ** Author: Sampo Kellomaki (sampo@iki.fi)
  ** This is confidential unpublished proprietary source code of the author.
  ** NO WARRANTY, not even implied warranties. Contains trade secrets.
- ** Distribution prohibited unless authorized in writing. See file COPYING.
- ** $Id: dec-templ.c,v 1.19 2006/09/30 06:24:49 sampo Exp $
+ ** Distribution prohibited unless authorized in writing.
+ ** Licensed under Apache License 2.0, see file COPYING.
+ ** $Id: dec-templ.c,v 1.23 2007-06-21 23:32:32 sampo Exp $
  **
  ** 28.5.2006, created, Sampo Kellomaki (sampo@iki.fi)
  ** 8.8.2006,  reworked namespace handling --Sampo
  ** 12.8.2006, added special scanning of xmlns to avoid backtracking elem recognition --Sampo
  ** 23.9.2006, added collection of WO information --Sampo
+ ** 21.6.2007, improved handling of undeclared namespace prefixes --Sampo
  **
  ** N.B: This template is meant to be processed by pd/xsd2sg.pl. Beware
  ** of special markers that xsd2sg.pl expects to find and understand.
@@ -21,6 +24,7 @@
 #define EL_NS     ELNS
 #define EL_TAG    ELTAG
 
+/* Called by: */
 struct ELSTRUCT* TXDEC_ELNAME(struct zx_ctx* c, struct zx_ns_s* ns ROOT_N_DECODE)
 {
   int tok;
@@ -42,7 +46,7 @@ struct ELSTRUCT* TXDEC_ELNAME(struct zx_ctx* c, struct zx_ns_s* ns ROOT_N_DECODE
 
   /* The tag name has already been detected. Process attributes until '>' */
   
-  for (; 1; ++c->p) {
+  for (; c->p; ++c->p) {
     ZX_SKIP_WS(c,x);
     if (ONE_OF_2(*c->p, '>', '/'))
       break;
@@ -88,11 +92,13 @@ set_attr_val:
 next_attr:
     continue;
   }
-  ++c->p;
-  if (c->p[-1] == '/' && c->p[0] == '>') {  /* Tag without content */
+  if (c->p) {
     ++c->p;
-    x->gg.g.err &= ~ZXERR_TAG_NOT_CLOSED;
-    goto out;
+    if (c->p[-1] == '/' && c->p[0] == '>') {  /* Tag without content */
+      ++c->p;
+      x->gg.g.err &= ~ZXERR_TAG_NOT_CLOSED;
+      goto out;
+    }
   }
 #endif
 
@@ -100,7 +106,7 @@ next_attr:
   
   ZX_START_BODY_DEC_EXT(x);
   
-  while (1) {
+  while (c->p) {
   next_elem:
     ZX_SKIP_WS(c,x);
     if (*c->p == '<') {
@@ -162,7 +168,7 @@ ELEMS;
     }
     /* Data */
     name = c->p;
-    ZX_LOOK_FOR(c,'<',x);
+    if (c->p) ZX_LOOK_FOR(c,'<',x);
     ss = ZX_ZALLOC(c, struct zx_str);
     ss->len = c->p - name;
     ss->s = name;
@@ -197,6 +203,7 @@ ELEMS;
  * One of each (attr and elem) is needed for every prefix used in code generation.
  * The ...2tok() functions come from code generation via gperf. */
 
+/* Called by:  TXDEC_ELNAME */
 int TXattr_lookup(struct zx_ctx* c, char* name, char* lim, struct zx_ns_s** ns)
 {
   const struct zx_tok* zt;
@@ -213,7 +220,7 @@ int TXattr_lookup(struct zx_ctx* c, char* name, char* lim, struct zx_ns_s** ns)
   if (!zt) {
     if (prefix && (name-1)-prefix == sizeof("xmlns")-1
 	&& !memcmp("xmlns", prefix, sizeof("xmlns")-1)) {
-      /* Namespace declaration. Skip because these were prescanned. */
+      /* Namespace declaration. Skip because these were prescanned (see ablec in this file). */
       return ZX_TOK_XMLNS;
     }
     return ZX_TOK_NOT_FOUND;
@@ -232,6 +239,7 @@ int TXattr_lookup(struct zx_ctx* c, char* name, char* lim, struct zx_ns_s** ns)
 
 /* FUNC(TXelem_lookup) */
 
+/* Called by:  TXDEC_ELNAME x2 */
 int TXelem_lookup(struct zx_ctx* c, char* name, char* lim, struct zx_ns_s** ns)
 {
   const struct zx_tok* zt;
