@@ -1,29 +1,32 @@
 /* zxutil.c  -  Utility functions
- * Copyright (c) 2006 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
+ * Copyright (c) 2006 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
- * Distribution prohibited unless authorized in writing. See file COPYING.
- * $Id: zxutil.c,v 1.12 2006/10/15 00:27:26 sampo Exp $
+ * Distribution prohibited unless authorized in writing.
+ * Licensed under Apache License 2.0, see file COPYING.
+ * $Id: zxutil.c,v 1.27 2007-06-21 23:32:32 sampo Exp $
  *
  * 15.4.2006, created over Easter holiday --Sampo
  */
 
 #include "errmac.h"
 
-#include <pthread.h>
+#include <stdarg.h>
+#include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
-#include <stdio.h>
+#include <time.h>
 #include <openssl/sha.h>
 #include <zlib.h>
 
 #include "zx.h"
 #include "zxidconf.h"
 
-#if !defined(DSSTDIO) && !defined(MINGW)
+#if !defined(USE_STDIO) && !defined(MINGW)
 /* *** Static initialization of struct flock is suspect since man fcntl() documentation
  * does not guarantee ordering of the fields, or that they would be the first fields.
  * On Linux-2.4 and 2.6 as well as Solaris-8 the ordering is as follows, but this needs
@@ -34,55 +37,72 @@ struct flock ds_wrlk = { F_WRLCK, SEEK_SET, 0, 0 };
 struct flock ds_unlk = { F_UNLCK, SEEK_SET, 0, 0 };
 #endif
 
-int name_from_path(char* buf, int buf_len, char* base, char* dir, char* name)
+/* Called by:  name_from_path, vopen_fd_from_path */
+int vname_from_path(char* buf, int buf_len, char* name_fmt, va_list ap)
 {
-  int base_len, dir_len, len, fd;
-  if (base) {
-    base_len = strlen(base);
-    if (base_len >= buf_len)
-      goto too_long;
-    memcpy(buf, base, base_len);
-  } else
-    base_len = 0;
-  if (dir) {
-    dir_len = strlen(dir);
-    if (base_len + dir_len >= buf_len)
-      goto too_long;
-    memcpy(buf + base_len, dir, dir_len);
-  } else
-    dir_len = 0;
-  len = strlen(name);
-  if (base_len + dir_len + len >= buf_len - 1)
-    goto too_long;
-  strcpy(buf + base_len + dir_len, name);
+  int len = vsnprintf(buf, buf_len, name_fmt, ap);
+  if (len < 0) {
+    perror("vsnprintf");
+    D("Broken vsnprintf? Impossible to compute length of string. Be sure to `export LANG=C' if you get errors about multibyte characters. Length returned: %d", len);
+    return 0;
+  }
   return 1;
-
+#if 0
  too_long:
   ERR("Path too long base(%s) dir(%s) name(%s)\n", base, dir, name);
   return 0;
+#endif
 }
 
-int open_fd_from_path(char* base, char* dir, char* name, int flags, int mode)
+/* Called by:  zxid_del_ses x3, zxid_find_epr, zxid_put_ses, zxlog x2 */
+int name_from_path(char* buf, int buf_len, char* name_fmt, ...)
+{
+  int ret;
+  va_list ap;
+  va_start(ap, name_fmt);
+  ret = vname_from_path(buf, buf_len, name_fmt, ap);
+  va_end(ap);
+  return ret;
+}
+
+/* Called by:  open_fd_from_path, read_all */
+int vopen_fd_from_path(int flags, int mode, char* logkey, char* name_fmt, va_list ap)
 {
   int fd;
   char buf[ZXID_MAX_BUF];
-  if (!name_from_path(buf, sizeof(buf), base, dir, name))
+  if (!vname_from_path(buf, sizeof(buf), name_fmt, ap))
     return -1;
   fd = open(buf, flags, mode);
   if (fd == -1) {
-    perror("open");
-    ERR("File(%s) not found.", buf);
+    if (logkey[0] != '-') {
+      perror("open (vopen_fd_from_path)");
+      ERR("File(%s) not found lk(%s). flags=0x%x", buf, logkey, flags);
+    }
     return -1;
   }
   return fd;
 }
 
-/* Low level function that keeps on sucking from a file descriptor until
- * want is satisfied or error happens. */
+/* Called by:  zxid_get_ent_from_file, zxid_get_meta, zxid_put_ses, zxid_write_ent_to_cache */
+int open_fd_from_path(int flags, int mode, char* logkey, char* name_fmt, ...)
+{
+  va_list ap;
+  int fd;
+  va_start(ap, name_fmt);
+  fd = vopen_fd_from_path(flags, mode, logkey, name_fmt, ap);
+  va_end(ap);
+  return fd;
+}
 
+/* Low level function that keeps on sucking from a file descriptor until
+ * want is satisfied or error happens.
+ * Return value reflects last got, i.e. what last read(2) system call returned.
+ * got_all reflects the total number of bytes received. */
+
+/* Called by:  main x3, opt x3, read_all, zxid_get_ent_from_file, zxid_simple_cf */
 int read_all_fd(int fd, char* p, int want, int* got_all)
 {
-#ifdef DSSTDIO
+#ifdef USE_STDIO
   int got;
   got = fread(p, 1, want, (FILE*)fd);
   if (got_all) *got_all = got;
@@ -91,20 +111,44 @@ int read_all_fd(int fd, char* p, int want, int* got_all)
   if (!ReadFile((HANDLE)fd, p, want, &got, 0))
     return -1;
   if (got_all) *got_all = got;
-#else
+#else  /* The Unix way */
   int got = 0;
   if (got_all) *got_all = 0;
   while (want) {
     got = read(fd, p, want);
     if (got <= 0) break;  /* EOF, possibly premature */
-    if (got_all) *got_all += got;
+    if (got_all)
+      *got_all += got;
     p += got;
     want -= got;
   }
 #endif
-  return got;
+  return got;   /* N.B. This is the last got, not the total. Use got_all for total. */
 }
 
+/* Returns actual total length. The buffer will always be nul terminated. */
+/* Called by:  opt x6, test_mode x2, zxid_conf_to_cf_len, zxid_find_epr, zxid_get_ses, zxid_get_ses_sso_a7n, zxid_read_cert, zxid_read_private_key, zxid_sha1_file */
+int read_all(int maxlen, char* buf, char* logkey, char* name_fmt, ...)
+{
+  va_list ap;
+  int fd, gotall;
+  va_start(ap, name_fmt);
+  fd = vopen_fd_from_path(O_RDONLY, 0, logkey, name_fmt, ap);
+  va_end(ap);
+  if (fd == -1) return 0;
+  if (read_all_fd(fd, buf, maxlen, &gotall) == -1) {
+    perror("Trouble reading.");
+    D("read error lk(%s)", logkey);
+    close(fd);
+    buf[maxlen-1] = 0;
+    return 0;
+  }
+  close(fd);
+  buf[MIN(gotall, maxlen-1)] = 0;  /* nul terminate */
+  return gotall;
+}
+
+/* Called by:  main, write2_or_append_lock_c_path x4, zxid_curl_write_data, zxid_put_ses, zxid_send_sp_meta, zxid_snarf_eprs_from_ses x2, zxid_write_ent_to_cache */
 int write_all_fd(int fd, char* p, int pending)
 {
 #ifdef MINGW
@@ -127,10 +171,14 @@ int write_all_fd(int fd, char* p, int pending)
   return 1;
 }
 
-int write_or_append_lock_c_path(char* c_path, char* data, int len, CU8* which, int seeky, int flag)
+/* Called by:  zxlog_blob, zxlog_write_line x2 */
+int write2_or_append_lock_c_path(char* c_path, int len1, char* data1, int len2, char* data2,
+				 CU8* which,  /* log key */
+				 int seeky,   /* SEEK_END, O_APPEND == append */
+				 int flag)    /* SEEK_SET, O_TRUNC  == overwrite */
 {
   int fd;
-  if (!c_path || !data)
+  if (!c_path)
     return 0;
 #ifdef MINGW
   fd = CreateFile(c_path, MINGW_RW_PERM, 0 /* 0  means no sharing allowed */, 0 /* security */,
@@ -142,11 +190,20 @@ int write_or_append_lock_c_path(char* c_path, char* data, int len, CU8* which, i
     SetFilePointer(fd, 0, &zero, FILE_END);  /* seek to end */
   }
 
-  if (!write_all_fd(fd, data, len)) {
-    ERR("%s: Writing to file `%s' %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len, errno, STRERROR(errno));
-    return 0;
+  if (len1 && data1) {
+    if (!write_all_fd(fd, data1, len1)) {
+      ERR("%s: Writing to file `%s' %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len1, errno, STRERROR(errno));
+      close(fd);
+      return 0;
+    }
+  }
+  if (len2 && data2) {
+    if (!write_all_fd(fd, data2, len2)) {
+      ERR("%s: Writing to file `%s' %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len2, errno, STRERROR(errno));
+      close(fd);
+      return 0;
+    }
   }
-  
 #else
   fd = open(c_path, O_WRONLY | O_CREAT | flag, 0666);
   if (fd == -1) goto badopen;
@@ -163,14 +220,30 @@ int write_or_append_lock_c_path(char* c_path, char* data, int len, CU8* which, i
   }
   
   lseek(fd,0,seeky);
-  if (!write_all_fd(fd, data, len)) {
-    ERR("%s: Writing to file(%s) %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len, errno, STRERROR(errno));
+  if (len1 && data1) {
+    if (!write_all_fd(fd, data1, len1)) {
+      ERR("%s: Writing to file(%s) %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len1, errno, STRERROR(errno));
 #ifdef USE_LOCK
-    flock(fd, LOCK_UN);
+      flock(fd, LOCK_UN);
 #else
-    lockf(fd, F_ULOCK, 0);
+      lockf(fd, F_ULOCK, 0);
 #endif
-    return 0;
+      close(fd);
+      return 0;
+    }
+  }
+
+  if (len2 && data2) {
+    if (!write_all_fd(fd, data2, len2)) {
+      ERR("%s: Writing to file(%s) %d bytes failed: %d %s. Check permissions and disk space.", which, c_path, len2, errno, STRERROR(errno));
+#ifdef USE_LOCK
+      flock(fd, LOCK_UN);
+#else
+      lockf(fd, F_ULOCK, 0);
+#endif
+      close(fd);
+      return 0;
+    }
   }
   
 #ifdef USE_LOCK
@@ -180,7 +253,7 @@ int write_or_append_lock_c_path(char* c_path, char* data, int len, CU8* which, i
 #endif
 #endif
   if (close(fd) < 0) {
-    ERR("%s: closing file(%s) after writing %d bytes failed: %d %s. Check permissions and disk space. Could be NFS problem.", which, c_path, len, errno, STRERROR(errno));
+    ERR("%s: closing file(%s) after write failed: %d %s. Check permissions and disk space. Could be NFS problem.", which, c_path, errno, STRERROR(errno));
     return 0;
   }
   return 1;
@@ -189,6 +262,7 @@ badopen:
   return 0;
 }
 
+/* Called by:  hexdmp, zxsig_data_rsa_sha1 x2, zxsig_verify_data_rsa_sha1 x3 */
 int hexdump(char* msg, char* p, char* lim, int max)
 {
   int i;
@@ -223,6 +297,11 @@ int hexdump(char* msg, char* p, char* lim, int max)
   return 0;
 }
 
+/* Called by:  zxsig_validate x5 */
+int hexdmp(char* msg, char* p, int len, int max) {
+  return hexdump(msg, p, p+len, max);
+}
+
 /*
  *  Base 64 encoding and decoding in its canonical and URL safe forms.
  */
@@ -328,7 +407,7 @@ unsigned char zx_std_index_64[256] = {
  * point to the same buffer. Both canonical and safe base64 are handled.
  * Returns pointer one past last output char written. */
 
-/* Called by:  unbase64 */
+/* Called by:  main x5, zxid_extract_cert, zxid_extract_private_key, zxid_process_keys, zxid_sp_deref_art, zxid_sp_dispatch, zxid_sp_dispatch_location, zxsig_validate x2 */
 char* unbase64_raw(char* p, char* lim, char* r, char* index_64)
 {
   int i;
@@ -344,8 +423,7 @@ char* unbase64_raw(char* p, char* lim, char* r, char* index_64)
       
       if (p == lim) {
 	if (i < 4) {
-	  if (i)
-	    ERR("Premature end of base64 data. (incomplete base64 input) i=%d\n", i);
+	  /*if (i) ERR("Premature end of base64 data. (incomplete base64 input) i=%d", i);*/
 	  if (i < 2) return r;
 	  if (i == 2) c[2] = Eq;
 	  c[3] = Eq;
@@ -355,7 +433,7 @@ char* unbase64_raw(char* p, char* lim, char* r, char* index_64)
     } while (i < 4);
     
     if (c[0] == Eq || c[1] == Eq) {
-      ERR("Premature end of base64 data. (incomplete base64 input) c0(%x)\n", c[0]);
+      ERR("Premature end of base64 data. (incomplete base64 input) c0(%x)", c[0]);
       break;
     }
     /* D("c0=%d,c1=%d,c2=%d,c3=%d\n", c[0],c[1],c[2],c[3]); */
@@ -369,26 +447,30 @@ char* unbase64_raw(char* p, char* lim, char* r, char* index_64)
   return r;
 }
 
-/* The out_buf should be 28+1 chars in length (+1 for nul termination). */
+/* The out_buf should be 28 chars in length. The buffer is not automatically nul termianated.
+ * There will be 27 characters of payload, plus one termination charater ".". */
 
-void sha1_safe_base64(char* out_buf, int len, char* data)
+/* Called by:  zxid_epr_path, zxid_get_ent_from_cache, zxid_parse_meta, zxlog, zxlog_path x2, zxlog_write_line */
+char* sha1_safe_base64(char* out_buf, int len, char* data)
 {
   char sha1[20];
   SHA1(data, len, sha1);
-  base64_fancy_raw(sha1, 20, out_buf, safe_basis_64, 1<<31, 0, 0, '.');
-  out_buf[28] = 0;
+  return base64_fancy_raw(sha1, 20, out_buf, safe_basis_64, 1<<31, 0, 0, '.');
 }
 
+/* Called by: */
 voidpf zx_zlib_zalloc(void* opaque, uInt items, uInt size)
 {
   return ZX_ALLOC(opaque, items*size);
 }
 
+/* Called by: */
 void zx_zlib_zfree(void* opaque, voidpf addr)
 {
   ZX_FREE(opaque, addr);
 }
 
+/* Called by:  zxid_saml2_redir_enc, zxlog_write_line */
 char* zx_zlib_raw_deflate(struct zx_ctx* c, int in_len, char* in, int* out_len)
 {
   int ret, dlen;
@@ -423,6 +505,7 @@ char* zx_zlib_raw_deflate(struct zx_ctx* c, int in_len, char* in, int* out_len)
   return out;
 }
 
+/* Called by:  zxid_sp_dispatch, zxid_sp_dispatch_location, zxlog_zsig_verify_print */
 char* zx_zlib_raw_inflate(struct zx_ctx* c, int in_len, char* in, int* out_len)
 {
   int ret, dlen, iter = 30;
@@ -489,32 +572,56 @@ char* zx_zlib_raw_inflate(struct zx_ctx* c, int in_len, char* in, int* out_len)
 #define URL_BAD(x) (((x)<=' ')||((x)>=0x7f)||ONE_OF_4((x),'+','%','=','&')||ONE_OF_2((x),'#','?'))
 #endif
 
-char* zx_url_encode(struct zx_ctx* c, int in_len, char* in, int* out_len)
+/* Called by:  zx_url_encode, zxid_saml2_redir_enc x2 */
+int zx_url_encode_len(int in_len, char* in)
 {
   int n;
-  char* out;
-  char* p;
   char* lim;
-  char* r;
   /* scan through to see how many escape expansions are needed */
-  p = in;
   lim = in + in_len;
-  for (n = 0; p < lim; ++p)
-    if (URL_BAD(*p)) ++n;
-  
-  p = in;
-  out = ZX_ALLOC(c, n+n+in_len+1);
-  *out_len = n+n+in_len;
-  
-  for (r = out; p < lim; ++p)
-    if (URL_BAD(*p)) {
-      *r++ = '%';
-      *r++ = HEX_DIGIT((*p >> 4) & 0x0f);
-      *r++ = HEX_DIGIT(*p & 0x0f);
+  for (n = 0; in < lim; ++in)
+    if (URL_BAD(*in)) ++n;
+  return n+n+in_len+1;   /* nul-terminated length */
+}
+
+/* Called by:  zx_url_encode, zxid_saml2_redir_enc x2 */
+void zx_url_encode_raw(int in_len, char* in, char* out)
+{
+  char* lim;
+  for (lim = in+in_len; in < lim; ++in)
+    if (URL_BAD(*in)) {
+      *out++ = '%';
+      *out++ = HEX_DIGIT((*in >> 4) & 0x0f);
+      *out++ = HEX_DIGIT(*in & 0x0f);
     } else
-      *r++ = *p;
-  *r = '\0';
+      *out++ = *in;
+}
+
+/* Called by: */
+char* zx_url_encode(struct zx_ctx* c, int in_len, char* in, int* out_len)
+{
+  char* out;
+  *out_len = zx_url_encode_len(in_len, in) + 1;
+  out = ZX_ALLOC(c, *out_len);
+  zx_url_encode_raw(in_len, in, out);
+  out[*out_len-1] = '\0';
   return out;
 }
 
+/* 01234567890123456789
+ * yyyy-MM-ddThh:mm:ssZ */
+
+/* Called by:  zxid_sp_sso_finalize x3 */
+int zx_date_time_to_secs(char* dt)
+{
+  struct tm t;
+  memset(&t, 0, sizeof(t));
+  sscanf(dt, "%d-%d-%dT%d:%d:%dZ",
+	 &t.tm_year, &t.tm_mon, &t.tm_mday,
+	 &t.tm_hour, &t.tm_min, &t.tm_sec);
+  t.tm_year -= 1900;
+  --t.tm_mon;
+  return mktime(&t);
+}
+
 /* EOF  --  zxutil.c */
